@@ -1,6 +1,6 @@
 # -*- coding: utf-8 -*-
 #
-# Copyright (C) 2012-2013 Vinay Sajip.
+# Copyright (C) 2012-2022 Vinay Sajip.
 # Licensed to the Python Software Foundation under a contributor agreement.
 # See LICENSE.txt and CONTRIBUTORS.txt.
 #
@@ -12,12 +12,16 @@ import subprocess
 import sys
 import tempfile
 import textwrap
+try:
+    import venv
+except ImportError:
+    venv = None
 
 from compat import unittest
+from support import DistlibTestCase
 
-from distlib import DistlibException
 from distlib.compat import fsencode, sysconfig
-from distlib.scripts import ScriptMaker, _enquote_executable
+from distlib.scripts import ScriptMaker, enquote_executable
 from distlib.util import get_executable
 
 HERE = os.path.abspath(os.path.dirname(__file__))
@@ -29,11 +33,11 @@ COPIED_SCRIPT = '''#!python
 MADE_SCRIPT = 'made = dummy.module:main'
 
 
-class ScriptTestCase(unittest.TestCase):
+class ScriptTestCase(DistlibTestCase):
 
     def setUp(self):
         source_dir = os.path.join(HERE, 'scripts')
-        target_dir = tempfile.mkdtemp()
+        target_dir = tempfile.mkdtemp(prefix='distlib-test-')
         self.maker = ScriptMaker(source_dir, target_dir, add_launchers=False)
 
     def tearDown(self):
@@ -56,9 +60,9 @@ class ScriptTestCase(unittest.TestCase):
                 self.assertIn(executable, first_line)
 
     def test_shebangs_custom_executable(self):
-        srcdir = tempfile.mkdtemp()
+        srcdir = tempfile.mkdtemp(prefix='distlib-test-')
         self.addCleanup(shutil.rmtree, srcdir)
-        dstdir = tempfile.mkdtemp()
+        dstdir = tempfile.mkdtemp(prefix='distlib-test-')
         self.addCleanup(shutil.rmtree, dstdir)
         maker = ScriptMaker(srcdir, dstdir, add_launchers=False)
         maker.executable = 'this_should_appear_in_the_shebang_line(中文)'
@@ -79,6 +83,47 @@ class ScriptTestCase(unittest.TestCase):
             actual = f.readline().decode('utf-8')
         self.assertIn(maker.executable, actual)
 
+
+    @unittest.skipIf(os.name != 'posix', 'Test only appropriate for '
+                     'POSIX systems')
+    def test_custom_shebang(self):
+        # Construct an executable with a space in it
+        self.maker.executable = 'an executable with spaces'
+        filenames = self.maker.make('script1.py')
+        with open(filenames[0], 'rb') as f:
+            first_line = f.readline()
+            second_line = f.readline()
+            third_line = f.readline()
+        self.assertEqual(first_line, b'#!/bin/sh\n')
+        self.assertEqual(second_line, b"'''exec' an executable with "
+                                      b'spaces "$0" "$@"\n')
+        self.assertEqual(third_line, b"' '''\n")
+        # Python 3.3 cannot create a venv in an existing directory
+        if venv and sys.version_info[:2] >= (3, 4):
+            if sys.platform == 'darwin':
+                # Supposedly 512, but various symlinks mean that temp folder
+                # names get larger than you'd expect ... might vary on different
+                # OS versions, too
+                dlen = 220
+            else:
+                dlen = 127
+            dstdir = tempfile.mkdtemp(suffix='cataaaaaa' + 'a' * dlen)
+            self.addCleanup(shutil.rmtree, dstdir)
+            bindir = os.path.join(dstdir, 'bin')
+            maker = ScriptMaker(self.maker.source_dir, bindir,
+                                add_launchers=False)
+            venv.create(dstdir)
+            maker.executable = os.path.join(bindir, 'python')
+            filenames = maker.make('script8.py')
+            p = subprocess.Popen(filenames[0], shell=True,
+                                 stdout=subprocess.PIPE, stderr=subprocess.PIPE)
+            stdout, stderr = p.communicate()
+            self.assertEqual(p.returncode, 0)
+            self.assertEqual(stderr, b'')
+            expected = os.path.realpath(maker.executable)  # symlinks on OS X
+            actual = os.path.realpath(stdout.strip())
+            self.assertEqual(actual, expected.encode('utf-8'))
+
     def test_multiple(self):
         specs = ('foo.py', 'script1.py', 'script2.py', 'script3.py',
                  'shell.sh', 'uwsgi_part')
@@ -91,34 +136,44 @@ class ScriptTestCase(unittest.TestCase):
 
     def test_generation(self):
         self.maker.clobber = True
-        for name in ('main', 'other_main'):
-            for options in (None, {}, {'gui': False}, {'gui': True}):
-                gui = options and options.get('gui', False)
-                spec = 'foo = foo:' + name
-                files = self.maker.make(spec, options)
-                self.assertEqual(len(files), 2)
-                actual = set()
-                for f in files:
-                    d, f = os.path.split(f)
-                    actual.add(f)
-                if os.name == 'nt':  # pragma: no cover
-                    if gui:
-                        ext = 'pyw'
+
+        def do_test(maker, version_info):
+            for name in ('main', 'other_main'):
+                for options in (None, {}, {'gui': False}, {'gui': True}):
+                    gui = options and options.get('gui', False)
+                    spec = 'foo = foo:' + name
+                    files = maker.make(spec, options)
+                    self.assertEqual(len(files), 2)
+                    actual = set()
+                    for f in files:
+                        d, f = os.path.split(f)
+                        actual.add(f)
+                    if os.name == 'nt':  # pragma: no cover
+                        if gui:
+                            ext = 'pyw'
+                        else:
+                            ext = 'py'
+                        expected = set(['foo.%s' % ext,
+                                        'foo-%s.%s.%s' % (version_info[0],
+                                                          version_info[1],
+                                                          ext)])
                     else:
-                        ext = 'py'
-                    expected = set(['foo.%s' % ext,
-                                    'foo-%s.%s' % (sys.version[:3], ext)])
-                else:
-                    expected = set(['foo', 'foo-%s' % sys.version[:3]])
-                self.assertEqual(actual, expected)
-                self.assertEqual(d, self.maker.target_dir)
-                for fn in files:
-                    with open(fn, 'r') as f:
-                        text = f.read()
-                    self.assertIn("_resolve('foo', '%s')" % name, text)
-                    if options and options['gui'] and os.name == 'nt':  # pragma: no cover
-                        first_line, rest = text.split('\n', 1)
-                        self.assertIn('pythonw', first_line)
+                        expected = set(['foo', 'foo-%s.%s' % (version_info[0],
+                                        version_info[1])])
+                    self.assertEqual(actual, expected)
+                    self.assertEqual(d, maker.target_dir)
+                    for fn in files:
+                        with open(fn, 'r') as f:
+                            text = f.read()
+                        # self.assertIn("_resolve('foo', '%s')" % name, text)
+                        if options and options['gui'] and os.name == 'nt':  # pragma: no cover
+                            first_line, rest = text.split('\n', 1)
+                            self.assertIn('pythonw', first_line)
+
+        do_test(self.maker, sys.version_info)
+        # See issue 134. Test for specifying the target Python version
+        self.maker.version_info = version_info = (4, 4)  # just any other Python version
+        do_test(self.maker, version_info)
 
     def test_clobber(self):
         files = self.maker.make('foo = foo:main')
@@ -169,9 +224,9 @@ class ScriptTestCase(unittest.TestCase):
 
     @unittest.skipIf(os.name != 'nt', 'Test is Windows-specific')
     def test_launcher_run_with_interpreter_args(self):
-        srcdir = tempfile.mkdtemp()
+        srcdir = tempfile.mkdtemp(prefix='distlib-test-')
         self.addCleanup(shutil.rmtree, srcdir)
-        dstdir = tempfile.mkdtemp()
+        dstdir = tempfile.mkdtemp(prefix='distlib-test-')
         self.addCleanup(shutil.rmtree, dstdir)
         maker = ScriptMaker(srcdir, dstdir, add_launchers=True)
 
@@ -201,8 +256,12 @@ class ScriptTestCase(unittest.TestCase):
         wlauncher = self.maker._get_launcher('w')
         tlauncher = self.maker._get_launcher('t')
         self.maker.add_launchers = True
-        executable = os.path.normcase(sys.executable).encode('utf-8')
-        wexecutable = executable.replace(b'python.exe', b'pythonw.exe')
+        # executable = os.path.normcase(sys.executable).encode('utf-8')
+        executable = sys.executable.encode('utf-8')
+        if b'python3.' in executable:
+            wexecutable = executable.replace(b'python3.', b'pythonw3.')
+        else:
+            wexecutable = executable.replace(b'python.', b'pythonw.')
         files = self.maker.make('script4.py')
         self.assertEqual(len(files), 1)
         filenames = set([os.path.basename(f) for f in files])
@@ -216,7 +275,7 @@ class ScriptTestCase(unittest.TestCase):
         files = self.maker.make('foo = foo:main', {'gui': True})
         self.assertEqual(len(files), 2)
         filenames = set([os.path.basename(f) for f in files])
-        specific = sys.version[:3]
+        specific = '%s.%s' % sys.version_info[:2]
         self.assertEqual(filenames, set(('foo.exe', 'foo-%s.exe' % specific)))
         for fn in files:
             with open(fn, 'rb') as f:
@@ -277,16 +336,22 @@ class ScriptTestCase(unittest.TestCase):
 
     @unittest.skipUnless(os.name == 'posix', 'Test only valid for POSIX')
     def test_mode(self):
-        self.maker.set_mode = False
-        files = self.maker.make('foo = foo:main')
-        self.assertEqual(len(files), 2)
-        for f in files:
-            self.assertIn(os.stat(f).st_mode & 0o7777, (0o644, 0o664))
-        self.maker.set_mode = True
-        files = self.maker.make('bar = bar:main')
-        self.assertEqual(len(files), 2)
-        for f in files:
-            self.assertIn(os.stat(f).st_mode & 0o7777, (0o755, 0o775))
+        # save test runner's original umask and ensure default 022
+        saved_umask = os.umask(0o022)
+        try:
+            self.maker.set_mode = False
+            files = self.maker.make('foo = foo:main')
+            self.assertEqual(len(files), 2)
+            for f in files:
+                self.assertIn(os.stat(f).st_mode & 0o7777, (0o644, 0o664))
+            self.maker.set_mode = True
+            files = self.maker.make('bar = bar:main')
+            self.assertEqual(len(files), 2)
+            for f in files:
+                self.assertIn(os.stat(f).st_mode & 0o7777, (0o755, 0o775))
+        finally:
+            # restore the test runner's original umask
+            os.umask(saved_umask)
 
     def test_interpreter_args(self):
         executable = fsencode(get_executable())
@@ -327,9 +392,48 @@ class ScriptTestCase(unittest.TestCase):
                 ('/usr/bin/env with spaces', '/usr/bin/env "with spaces"'),
                 ('/usr/bin/env "pre spaced"', '/usr/bin/env "pre spaced"')
                 ):
-            self.assertEqual(_enquote_executable(executable),
-                             expected)
+            self.assertEqual(enquote_executable(executable), expected)
 
+    def test_variant_configuration(self):
+        spec = 'foo = foo:main'
+        files = self.maker.make(spec)
+        if os.name == 'nt':
+            ext = '.py'
+        else:
+            ext = ''
+        basenames = ('foo%s' % ext,
+                     'foo-%s.%s%s' % (self.maker.version_info[:2] + (ext,)))
+        expected = set([os.path.join(self.maker.target_dir, s) for s in basenames])
+        self.assertEqual(expected, set(files))
+        self.maker.variant_separator = ''
+        self.maker.clobber = True
+        files = self.maker.make(spec)
+        basenames = ('foo%s' % ext,
+                     'foo%s.%s%s' % (self.maker.version_info[:2] + (ext,)))
+        expected = set([os.path.join(self.maker.target_dir, s) for s in basenames])
+        self.assertEqual(expected, set(files))
+
+        class CustomMaker(ScriptMaker):
+            def get_script_filenames(self, name):
+                result = super(CustomMaker, self).get_script_filenames(name)
+                if 'X.Y' in self.variants:
+                    result.add('%s%s.%s' % ((name,) + self.version_info[:2]))
+                return result
+
+        maker = CustomMaker(self.maker.source_dir, self.maker.target_dir,
+                            add_launchers=False)
+        maker.clobber = True
+        files = maker.make(spec)
+        basenames = ('foo%s' % ext,
+                     'foo%s.%s%s' % (self.maker.version_info[:2] + (ext,)),
+                     'foo-%s.%s%s' % (self.maker.version_info[:2] + (ext,)))
+        expected = set([os.path.join(self.maker.target_dir, s) for s in basenames])
+        self.assertEqual(expected, set(files))
+        if os.name == 'nt':
+            maker.add_launchers = True
+            files = maker.make(spec)
+            expected = set([e.replace('.py', '.exe') for e in expected])
+            self.assertEqual(expected, set(files))
 
 if __name__ == '__main__':  # pragma: no cover
     unittest.main()
